package client

import (
	"net/http"
	"random.chars.jp/git/image-board/api"
	"random.chars.jp/git/image-board/store"
	"strings"
)

// User returns api.UserPayload of a snowflake.
func (r *Remote) User(flake string) (api.UserPayload, error) {
	var user api.UserPayload
	err := r.fetch(http.MethodGet, api.User+"/"+flake, &user, nil)
	return user, err
}

// This returns api.UserPayload currently authenticated as.
func (r *Remote) This() (api.UserPayload, error) {
	return r.User("this")
}

// Username returns api.UserPayload of username.
func (r *Remote) Username(name string) (api.UserPayload, error) {
	var user api.UserPayload
	err := r.fetch(http.MethodGet, api.Username+"/"+name, &user, nil)
	return user, err
}

// Auth authenticates using a username and its corresponding password.
func (r *Remote) Auth(username, password string) (string, error) {
	var secret api.UserSecretPayload
	err := r.fetch(http.MethodPost,
		populateField(api.UsernameAuth, "name", username),
		&secret,
		strings.NewReader(password))
	return secret.Secret, err
}

// UserAdd adds a new user.
func (r *Remote) UserAdd(username string, password string, privileged bool) (store.User, error) {
	if resp, err := r.requestJSON(http.MethodPut, api.User, api.UserCreatePayload{
		Username:   username,
		Password:   password,
		Privileged: privileged,
	}); err != nil {
		return store.User{}, err
	} else {
		var user store.User
		err = unmarshal(resp.Body, &user)
		return user, err
	}
}

// UserUpdate updates a user.
func (r *Remote) UserUpdate(flake, newname string) error {
	return r.requestJSONnoResp(http.MethodPatch,
		populateField(api.UserField, "flake", flake),
		api.UserUpdatePayload{Username: newname})
}

// UserDestroy destroys a user.
func (r *Remote) UserDestroy(flake string) error {
	return r.requestJSONnoResp(http.MethodDelete,
		populateField(api.UserField, "flake", flake),
		nil)
}

// UserDisable disables a user.
func (r *Remote) UserDisable(flake string) error {
	return r.requestJSONnoResp(http.MethodDelete,
		populateField(api.UserPassword, "flake", flake),
		nil)
}

// UserPassword sets a user's password.
func (r *Remote) UserPassword(flake, password string) (string, error) {
	if resp, err := r.request(http.MethodPut,
		populateField(api.UserPassword, "flake", flake),
		strings.NewReader(password)); err != nil {
		return "", err
	} else {
		var payload api.UserSecretPayload
		err = unmarshal(resp.Body, &payload)
		return payload.Secret, err
	}
}

// UserSecret returns a user's secret.
func (r *Remote) UserSecret(flake string) (string, error) {
	var secret api.UserSecretPayload
	err := r.fetch(http.MethodGet, populateField(api.UserSecret, "flake", flake), &secret, nil)
	return secret.Secret, err
}

// UserSecretRegen regenerates a user's secret.
func (r *Remote) UserSecretRegen(flake string) (string, error) {
	var secret api.UserSecretPayload
	err := r.fetch(http.MethodPut, populateField(api.UserSecret, "flake", flake), &secret, nil)
	return secret.Secret, err
}

// UserImages returns a slice of snowflakes of a user's images.
func (r *Remote) UserImages(flake string) ([]string, error) {
	var flakes []string
	err := r.fetch(http.MethodGet, populateField(api.UserImage, "flake", flake), &flakes, nil)
	return flakes, err
}
